from __version__ import version
from model import Project, Entity
from optparse import OptionParser
from report import Report
from utils  import getClass
from config import PROFILE_NS
import getopt # see below
import registry
import sys
import pkg_resources

profiles = pkg_resources.iter_entry_points(PROFILE_NS)
_marker = object()

class Engine(Entity):
    def __init__(self):
        Entity.__init__(self, None)
        self.registry = registry.Registry()
        self.parser = OptionParser(prog=registry.CONFIG_ROOT,
                                   version=version.v_long)
        self.report = Report()
        self.commands = {}
        self.plugins = {}
        self.project = None

        self.bootstrap()

    def _getKeyspace(self):
        return "/%s/world" % ( registry.CONFIG_ROOT)

    keyspace = property(_getKeyspace)

    def bootstrap(self):
        # load up the existing registry
        self.registry.bootstrap()

        # use a static option parser to search for an
        # optional config file
        parser = self.parser
        parser.add_option('-C', "--config", dest="configfile",
                          default=None,
                          help="Specify a configuration file")
        parser.add_option('-P', "--profile", dest="profile",
                             default="default",
                             help="Specify a profile file")

        # We use getopt to parse for these two args
        # we don't want it to implicitly term on --help
        # or generate premature usage (before plugins)
        configfile = None
        profile    = "default"
        try:
            optlist, args = getopt.gnu_getopt(sys.argv[1:],
                                              'C:P:',
                                              ["configfile=",
                                               "profile="])
            for o, arg in optlist:
                if o in ("-C", "--configfile"):
                    configfile = arg
                elif o in ("-P", "--profile"):
                    profile = a
        except getopt.GetoptError:
            pass

        # load optional config file
        if configfile:
            self.registry.load(configfile)

        # XXX add dialogue for adding commands / etc
        profmap = dict([(x.name, x) for x in profiles])
        profile = profmap.get(profile, _marker)
        if profile is _marker:
            print "profile not found, using default"
        profile = profmap.get('default')
        
        pxml = profile.load()
        self.registry.loadString(pxml())

        xp = "/%s/profiles/profile[@name='%s']/plugin" %(registry.CONFIG_ROOT, profile.name)
        plugins = self.registry.select(xp)

        # initialize the plugins
        for plug in plugins:
            get = plug.get
            name = get("name")
            klass = get("class", get("name"))
            data = get("data", get("name"))
            self.addPlugin(self.loadPlugin(klass, data), name)

        usage = self.compileCommands()
        self.parser.set_usage(usage)

    def addPlugin(self, plugin, name):
        if isinstance(plugin, type):
            plugin = plugin(self)
        plugin.options(self.parser)
        plugin.commands(self.commands)
        self.plugins[name] = plugin

    def loadPlugin(self, name, data):
        plugin = getClass(name, data)
        return plugin(self)

    def compileCommands(self):
        """returns a usage string construted from each command"""
        usage = ["%prog: command args", '']
        for key, command in self.commands.items():
            usage.append("%-15s -- %s" %(key, command.docs))
        return '\n'.join(usage)

    def run(self):
        # We now parse with all the plugins loaded
        # and make the options available to plugins
        (options, args) = self.parser.parse_args()
        self.options = options

        if not args or args[0] not in self.commands:
            self.parser.error("Unknown or no command")

        # the project name should come from the arguments
        self.project = Project(self)
        result = self.commands[args[0]](self.project, *args[1:])

        target = self.options.configfile or registry.CONFIG_FILE
        self.registry.save(target)
        return result

def run_engine():
    Engine().run()

