from config import *
from question import Question
from model import Entity
from utils import normalizePath
import os


class Command(object):
    """Plugins add new commands to the engine.
    When we get these on the command line we pass the rest of the
    options to them to trigger the proper action
    """
    def __init__(self, name, callback, docs=''):
        self.name = name
        self.callback = callback
        self.docs = docs
        if not docs: self.docs = callback.__doc__

    def __call__(self, *args): self.callback(*args)

class Plugin(Entity):
    """Plugins interpret and transform  the model. They do this by
    taking the exiting Entity model from 'model.py' and create/modifiy
    nodes in the output model.
    """
    def __init__(self, engine):
        Entity.__init__(self, engine)
        self.engine = engine
        self.registry = engine.registry
        self.questions = []
        self.templates = {}
        self.slots = set()
        self.fillBy = {}
        self.scope = PLUGIN
        self._datadir=''
        
    def __hash__(self):
        return hash(self.__name__)

    def _getKeyspace(self):
        return "/%s/plugins/plugin[@name='%s']" % ( CONFIG_ROOT,
                                                    self.name
                                                    )
    keyspace = property(_getKeyspace)

    def addQuestion(self, *args, **kw):
        question = Question(self, *args, **kw)
        self.questions.append(question)
        if not question.isCallback and question.action:
            self.fillBy[question.action] = question
        return question

    def _gatherSlots(self, tmpl):
        slots = tmpl.gatherSlots()
        if slots: self.slots.update(slots.keys())
        return slots

    def addTemplate(self, name, tmpl, depth=0):
        self._gatherSlots(tmpl)
        # make name project relative
        if depth:
            parts = name.split(os.path.sep)
            name = os.path.sep.join(parts[depth:])
        self.templates[name] = tmpl


    def addTemplateAt(self, tmpl, relativePath):
        """Where relative path is in the target project. Typically
        used to add resources managed by support plugins.
        """
        path = normalizePath(relativePath, self['base'], 0)
        tmpl.properties['target'] = path
        self.addTemplate(relativePath, tmpl)
        return path

    def fillSlotAt(self, relativePath, slotname, fragment):
        # The new fragment will need to have its slots processed
        slots = self._gatherSlots(fragment)
        # Late binding questions
        for s in slots:
            if s in self.fillBy:
                self.fillBy[s].ask()

        tmpl = self.templates[relativePath]
        tmpl.getSlotByName(slotname).fill(fragment)

    def options(self, optparser):
        """ Given an opt parser register a set of callback options
        pointing to self.

        parser.add_option("-c", "--check",
                          action="callback",
                          callback=my_callback)

        def my_callback(option, opt, value, parser)
        """

        pass

    def commands(self, commands):
        """Add Commands to the commands dict."""
        pass

    # Plugins typically go through a number of phases
    # They consume any stored project information
    # They read any plugin specfic data
    # Gather data not present from the user
    # They transform any objects in the system they care about
    # Plugins can override __call__ as needed, but by default
    # this lifecycle is enforced in a series of phases
    def __call__(self, project, *args):
        """Transform the project into the output DOM"""
        self.loadConfig(project)
        self.loadData(project)
        self.gatherAnswers(project)
        self.transform(project)

    def loadConfig(self, project):
        # Now we know enough to load the configuration and populate
        # the config data of plugins
        pass

    def loadData(self, project):
        pass

    def gatherAnswers(self, project):
        # Cross ref slots to filled by and ask away
        for q in self.questions:
            if q.isCallback or q.action in self.slots:
                q.ask()

    def transform(self, project):
        pass





