from skeletor.plugin import Plugin, Command
from skeletor.model import Project
from skeletor.utils import getModule, cloneTree, walkPluginDir
from skeletor.question import GLOBAL, PROJECT, PLUGIN, TEMPORARY
import skeletor.widgets as widgets
import skeletor.template as template
import datetime
import os
import pwd

extension_map = {'py': ('python', 'code', template.PythonTemplate),
                 'cpy': ('python', 'code', template.PythonTemplate),
                 'vpy': ('python', 'code', template.PythonTemplate),
                 'xml': ('xml', 'config', template.XMLTemplate),
                 'zcml': ('xml', 'config', template.XMLTemplate),
                 'pt': ('template', 'skin', template.XMLTemplate),
                 'zpt': ('template', 'skin', template.XMLTemplate),
                 'cpt': ('template', 'skin', template.XMLTemplate),
                 '': ('other', '', template.FileTemplate),
                 None: ('other', '', template.FileTemplate)
                }


class Base(Plugin):
    """Base model handler"""
    __name__= "base"

    def options(self, parser):
        parser.add_option('-p', '--project',
                          type="string",
                          action="store",
                          dest="ProjectName",
                          default="",
                          help="set the project name"
                          )

        parser.add_option('-d', '--directory',
                          type="string",
                          action="store",
                          dest="TargetDir",
                         help="the target directory we render into"
                          )

        parser.add_option("-c", "--console",
                          action="store_const",
                          const="console",
                          dest="display",
                          default="")

        parser.add_option("-r", "--regenerate",
                          action="store_true",
                          dest="regenerate",
                          help="""force regeneration of existing
                          project. This overwrites changes""",
                          default=False)


    def commands(self, commands):
        commands['create'] = Command('create', self.create,
                 """[project name] create a new base project and a model for the engine""")

    def loadData(self, project):
        ### Force a read from the plugin dir down
        ### XXX: this cannot be hard coded
        # Make sure there is a registered entry in the
        # target directory. This means checking the target dir
        # and using it  (unless flagged otherwise)
        # or the one in the template dir if not.
        base = self.registry.resolve('base')
        for kind, full, target in walkPluginDir(self, base):
            if kind != "file": continue
            ext = os.path.splitext(full)[1]
            if ext.startswith('.'):
                ext = ext[1:]
            (file_type, file_intent, tp) = \
                    extension_map.get(ext, extension_map.get(None))

            # use the target by default
            source = target
            if self.engine.options.regenerate or not os.path.exists(target):
                # or the orig template if needed
                source = full
            data = open(source, "r").read()
            t = tp(data,
                   file_name=full,
                   file_type=file_type,
                   file_intent=file_intent,
                   target=target)
            self.addTemplate(full, t, depth=2)

    def setProductName(self, value, question):
        # callback must set project.name
        self.engine.project.name = value
        question.record(question.actionname, value)
        self.registry.setScope(PROJECT, self.engine.project.keyspace)

    def loadConfig(self, project):
        self.addQuestion(widgets.entry,
                         "Project Name?",
                         self.setProductName,
                         flag="ProjectName",
                         actionname="ProductName").ask()

        self.addQuestion(widgets.fileselect,
                         "Target Directory",
                         "TargetDir",
                         flag="TargetDir",
                         default=os.path.join(os.path.abspath(os.curdir), ''),
                         directory=True).ask()

        path = os.path.join(
            os.path.abspath(self.registry.resolve('TargetDir')),
            self.engine.options.ProjectName)

        self.registry.set('base', path)

        userid=pwd.getpwuid(os.geteuid()).pw_name
        
        self.addQuestion(widgets.entry,
                         "Product Description?",
                         "ProductDescription",
                         scope=PROJECT)

        self.addQuestion(widgets.entry,
                         "Author",
                         "ProductAuthors",
                         default=userid,
                         scope=GLOBAL)

        self.addQuestion(widgets.entry,
                         "Email Address",
                         "ProductAuthorEmail",
                         default="%s@" %userid,
                         scope=GLOBAL)

        self.addQuestion(widgets.checkbox, "DocFormat", "DocFormat",
                         options=["text/plain", "text/restructured"],
                         column="Text Format",
                         scope=GLOBAL,
                         default="text/restructured",
                         single=True)

        self.addQuestion(widgets.entry, "Product Version", "ProductVersion",
                         default="0.1",
                         scope=PROJECT,
                         )

        # Load in the date for things that need it
        now = datetime.datetime.now()
        date = now.strftime("%m/%d/%Y")
        self.registry.set("Date", date, scope=TEMPORARY)

    def transform(self, project):
        # Fragments live in the plugins themselves
        # We can close the entire tree
        cloneTree(self.registry.resolve("base"), self)
        for ct in self.templates.values():
            self.engine.project.render(self, ct)

    def create(self, project, *args):
        if len(args):
            if args[0].count(os.path.sep):
                path = args[0].split(os.path.sep)
                self.engine.options.ProjectName = path[-1]
                self.engine.project.name = path[-1]
                self.engine.options.TargetDir = os.path.sep.join(path[:-1])
            else:
                self.engine.options.ProjectName = args[0]
                self.engine.project.name = args[0]
            args = args[1:]
            self.registry.setScope(PROJECT, self.engine.project.keyspace)
        self(self.engine.project, *args)

