import re
import os
import pkg_resources as pkgr
from copy import deepcopy
from config import *

try:
    import lxml.etree as ET
except ImportError:
    raise SystemExit("""You must have lxml installed""")

try:
    import xml.dom.minidom
    import xml.dom.ext
except ImportError:
    # Optional dep for pretty printing
    pass

parseXPath = re.compile(r'(?P<scope>\w+)\[@name=(?P<q>\')(?P<expr>[A-Za-z]\w*)(?P=q)\]')
BASECONFIG = """
<skeletor>
   <profiles>
   </profiles>
   <world/>
   <projects/>
   <plugins/>
   <temporary/>
</skeletor>
"""

class Registry(object):
    def __init__(self):
        self.root = ET.XML(BASECONFIG)
        self.tree = ET.ElementTree(self.root)
        self._scopes = {}

    def bootstrap(self):
        # Make a best effort to create a working skeletor
        # config environment
        if not os.path.exists(CONFIG_DIR):
            os.mkdir(CONFIG_DIR)
        if not os.path.isdir(CONFIG_DIR):
            raise IOError("cannot create %s directory, check config" % CONFIG_DIR)
        pkgr.set_extraction_path(os.path.dirname(CONFIG_FILE))
        if not os.path.exists(CONFIG_FILE):
            self.save(CONFIG_FILE)
        else:
            self.load(CONFIG_FILE)

    def _getScopedNames(self, key, scopes=None):
        if scopes == None: scopes = SCOPE_PRECEDENCE
        if isinstance(scopes, basestring): scopes = (scopes,)
        yield key
        for s in scopes:
            yield "/%s/%s" %(self.getScope(s), key)


    def getScope(self, key):
        return self._scopes.get(key, "%s/%s" %(CONFIG_ROOT, key))

    def setScope(self, scope, target):
        if target.startswith("/"): target = target[1:]
        self._scopes[scope] = target

    def setProject(self, ks): self.setScope(PROJECT, ks)
    def setPlugin(self, ks): self.setScope(PLUGIN, ks)


    def select(self, key, scope=None):
        # might have to support only element tree and
        # limit xpath internally
        if key.startswith('/'):
            return iter(self.root.xpath(key))
        elem = self.root
        for path in self._getScopedNames(key, scope):
            lst = elem.xpath(path)
            if lst: return iter(lst)
        return None

    def _resolve(self, key, scope=None):
        res = self.select(key, scope)
        # there should only be one match on a resolve call
        # if there is more than one we raise an error
        if not(res): raise KeyError("Nothing matched %s" %(key))
        res = list(res)
        if len(res) > 1: raise ValueError("resolve yields multiple values")
        return res[0]


    def resolve(self, key, scope=None):
        elem = self._resolve(key, scope)
        if elem is not None:
            return elem.text
        return None

    def get(self, key, default=None, scope=None):
        try:
            value = self.resolve(key, scope)
        except KeyError:
            value = default
        return value

    def set(self, key, value, scope=TEMPORARY, **kw):
        # no scope == temp
        # try to resolve the scope first, then set/replace key

        # The scope itself might not exist in which case we
        # will create the last element of it
        scope = "/" + self.getScope(scope)
        try:
            elem = self._resolve(scope)
        except KeyError:
            path, kind = os.path.split(scope)
            match = parseXPath.match(kind)
            if not match:
                raise Exception("No match for '%s: {%s:%s} %s'" %(kind, key, value, scope))
            kind, name = match.group('scope'), match.group('expr')

            target = self._resolve(path)
            if target is None:
                raise ValueError("No scope %s" % (
                    self.getScope(scope)))
            # create the element with the proper attrs
            elem = ET.SubElement(target, kind, name=name)

        # resolve the key (this also may or may not exist)
        try:
            elem = self._resolve(key)
        except KeyError:
            elem = ET.SubElement(elem, key, kw)

        if elem is not None:
            elem.text = value
        else:
            raise KeyError("unable to set value, no context %s/%s" %
                           (scope, key))

    def load(self, *files):
        for source in files:
            elem = ET.parse(source).getroot()
            if not elem: continue
            self.tag(elem, origin=source)
            self.merge(elem)

    def loadString(self, *fragments):
        for source in fragments:
            elem = ET.XML(source)
            if not elem: continue
            self.merge(elem)

    def tag(self, element, **kwargs):
        # Tag each element in the hierarchy with an
        # attribute
        items = kwargs.items()
        if not items: return

        for elem in element.getiterator():
            keys = elem.keys()
            for k,v in items:
                if k not in keys:
                    elem.set(k, v)

    def remove(self, element, *anames):
        for elem in element.getiterator():
            attrs = elem.attrib
            for name in anames:
                if name in attrs:
                    del attrs[name]

    def merge(self, element):
        # Merge a new element tree into the root
        # we do a top down merge letting each successive element
        # override existing nodes where possible
        root = self.root
        self._merge(root, element)

    def _merge(self, dest, source):
        # merge target into source, recursively we use @name as the
        # merge key, falling back to tagname
        getTag = lambda e: (e.tag, e.get('name', None,), e)

        sdata = [getTag(i) for i in list(source)]
        tdata = [getTag(i) for i in list(dest)]

        skeys = set([(i[0], i[1]) for i in sdata])
        tkeys = set([(i[0], i[1]) for i in tdata])
        sd = {}
        td = {}
        for t, n, e in sdata: sd[(t,n)] = e
        for t, n, e in tdata: td[(t,n)] = e

        # do set operations on each
        # its only intersects that require real work
        for tag, name in skeys.difference(tkeys):
            # there are adds
            dest.append(sd[(tag, name)])

        # now merge the set union
        for tag, name in skeys.intersection(tkeys):
            # merge the attr space, overwriting from source
            e = sd[(tag, name)]
            orig = td[(tag,name)]
            for k in e.keys(): orig.set(k, e.get(k))
            self._merge(orig, e)

    def __str__(self):
        root = deepcopy(self.root)
        self.remove(root, "origin")
        tmp  = root.find("temporary")
        root.remove(tmp)
        return ET.tostring(root, "utf-8")

    def save(self, filename):
        # silly way to pretty print
        data = str(self)
        fp = open(filename, "w")
        try:
            dom = xml.dom.minidom.parseString(data)
            xml.dom.ext.PrettyPrint(dom, fp)
        except:
            print >>fp, data
        fp.close()

