import compiler
import re
from xml.dom.minidom import parseString

SLOT_TYPE = ('code','text')
SLOT_TYPE_DEFAULT = 'text'

SLOT_REPLACEMENT = ('insertAfter', 'insertBefore', 'replace')
SLOT_REPLACEMENT_DEFAULT = 'replace'

RE_SLOT_BASE = '\s*(?P<slot_name>\w+)\s*(,\s*(?P<slot_type>%s)\s*)?(,\s*(?P<slot_replacement>%s)\s*)?' % \
               ('|'.join(SLOT_TYPE), '|'.join(SLOT_REPLACEMENT))
RE_WHITESPACE = re.compile('\s*')



def dfs(node, children=iter):
    for child in children(node):
        for c in dfs(child, children):
            yield c
    yield node


class BaseNode(object):
    slot_delimiters = ('#{', '}#')

    def __init__(self, **kwargs):
        self.properties = kwargs
        self.parent = None
        self.children = []

    def insertBefore(self, node):
        """Add a node to this node's parent immediately before this node"""
        index = self.parent.children.index(self)
        self.parent.children.insert(index, node)

    def insertAfter(self, node):
        """Add a node to this node's parent immediately after this node"""
        index = self.parent.children.index(self)
        self.parent.children.insert(index+1, node)

    def replace(self, node):
        """Replace this node in the parent's child list with the given node"""
        index = self.parent.children.index(self)
        self.parent.children[index] = node

    def appendChild(self, node):
        """Add a new child to this node"""
        node.parent = self
        self.children.append(node)

    def __iter__(self):
        """Return an iterator over this node's children"""
        return iter(self.children)

    def gatherSlots(self):
        """Return a set containing the child slots of this node"""
        slots = {}
        for c in self:
            if isinstance(c, Slot):
                slots[c.getName()] = c
        return slots



class FileTemplate(BaseNode):
    comment_style = ("#", "")
    # default slot is delimited like so: #{myslot}#
    re_slot = re.compile('#\{%s\}#' % RE_SLOT_BASE)

    def __init__(self, data, file_name=None, file_type=None,
                 file_intent=None, **kw):
        BaseNode.__init__(self, file_name=file_name,
                          file_type=file_type,
                          file_intent=file_intent, **kw)
        self.parseTemplate(data)

    def getSlotByName(self, slot_name):
        """Return a slot object with the specified name.  Returns
        None if no slot found"""
        for c in self.children:
            if c.properties.get('slot_name', None) == slot_name:
                return c

    def getFileName(self):
        return self.properties['file_name']

    def getFileType(self):
        return self.properties['file_type']

    def getFileIntent(self):
        return self.properties['file_intent']

    def render(self, dict, out):
        for c in self.children:
            c.render(dict, out)
        #self.validate(out.getvalue())

    def validate(self, s):
        pass

    def parseTemplate(self, s):
        start = 0
        for match in self.re_slot.finditer(s):
            if match.start():
                self.appendChild(TextNode(s[start:match.start()]))

            d = match.groupdict()
            g = d.get
            slot_name = d['slot_name']
            assert slot_name is not None

            slot_type = g('slot_type')
            if slot_type is None:
                slot_type = SLOT_TYPE_DEFAULT

            slot_replacement = g('slot_replacement')
            if slot_replacement is None:
                slot_replacement = SLOT_REPLACEMENT_DEFAULT

            slot_indentation = None
            if slot_type == 'code':
                line_start = s[0:match.start()].rfind('\n')+1
                # extract the line in which the current slot lives
                line = s[line_start:match.start()]
                slot_indentation = line[:RE_WHITESPACE.match(line).end()]

            self.appendChild(Slot(slot_name,
                                  slot_type,
                                  slot_replacement,
                                  slot_indentation,
                                  self.getSlotDelimiters()))
            start = match.end()

        self.appendChild(TextNode(s[start:]))

    def getSlotDelimiters(self):
        return self.slot_delimiters

    def __str__(self):
        return '<%s:%s>' % (self.__class__.__name__, self.getFileName())


class PythonTemplate(FileTemplate):
    # python files use the default slot
    comment_style = ("#", "")
    def validate(self, code):
        try:
            compiler.parse(code)
        except:
            print('Bad code fragment %s' % (code,))
            raise


class XMLTemplate(FileTemplate):
    # slots in xml look like xml comments <!--SLOT:myslot-->
    comment_style = ("<!--", "-->")
    slot_delimiters = ('<!--SLOT:','-->')
    re_slot = re.compile('<!--SLOT:%s-->' % RE_SLOT_BASE)

    def __init__(self, code, file_name, file_type, file_intent, **kw):
        FileTemplate.__init__(self, code, file_name=file_name,
                              file_type=file_type,
                              file_intent=file_intent, **kw)

    def validate(self, code):
        try:
            parseString(code)
        except:
            print('Bad xml fragment %s' % (code,))
            raise


class TextNode(BaseNode):
    def __init__(self, text, **kw):
        BaseNode.__init__(self, text=text, **kw)

    def getText(self):
        return self.properties['text']

    def render(self, dict, out):
        out.write(self.getText())

    def __str__(self):
        return self.getText()

    def __repr__(self): return str(self)


class Slot(BaseNode):
    def __init__(self, slot_name, slot_type='text',
                 slot_replacement='replace',
                 slot_indentation='', slot_delimiters=('#{','}#'), **kw):
        BaseNode.__init__(self, slot_name=slot_name,
                          slot_type=slot_type,
                          slot_replacement=slot_replacement,
                          slot_indentation=slot_indentation,
                          slot_delimiters=slot_delimiters,
                          **kw)

    def getName(self):
        return self.properties['slot_name']

    def getType(self):
        return self.properties['slot_type']

    def getReplacement(self):
        return self.properties['slot_replacement']

    def getIndentation(self):
        return self.properties['slot_indentation']

    def getDelimiters(self):
        return self.properties['slot_delimiters']

    def render(self, dict, out):
        if self.getReplacement() == 'insertAfter':
            out.write(str(self))
        v = dict.get(self.getName(), None)
        if v is None:
            # don't replace "replace" slots if there is no suitable keyword in dict
            if self.getReplacement() == 'replace':
                out.write(str(self))
        else:
            if hasattr(v, 'render'):
                v = v.render(dict, out)
            else:
                v = str(v)
            if self.getType() == 'code':
                # break into lines
                v = v.split('\n')
                # indent
                for i in range(0,len(v)):
                    if i > 0:
                        v[i] = self.getIndentation() + v[i]
                v = '\n'.join(v)
            out.write(v)
        if self.getReplacement() == 'insertBefore':
            out.write(str(self))


    def fill(self, node):
        rep = self.getReplacement()
        if rep == 'insertAfter':
            self.insertAfter(node)
        elif rep == 'insertBefore':
            self.insertBefore(node)
        else:
            self.replace(node)

    def __str__(self):
        return '%s%s, %s, %s%s' % (self.getDelimiters()[0],
                                   self.getName(),
                                   self.getType(),
                                   self.getReplacement(),
                                   self.getDelimiters()[1])

    def __repr__(self): return "<%s>" % self.properties['slot_name']

