import StringIO
import unittest
import template


code = """#{Header}#
\"\"\"
#{ProductName, replace}#

#{ProductDescription, replace}#

$Id: $
\"\"\"

__authors__ = '#{ProductAuthors}#',
__docformat__ = '#{DocFormat, replace}#',

#{ModuleLevelCode, code, insertAfter}#

def initialize(context):
    #{InitializeBegin, code, insertAfter}#
    #{InitializeEnd, code, insertBefore}#
    pass

"""

xml = """
<myxml>
  <sometag />
  <block>
    <!--SLOT:myslot1, code, insertAfter-->
    <!--SLOT:myslot2, code, replace-->
  </block>
</myxml>
"""


slots = (('Header', 'text', 'replace', None),
         ('ProductName', 'text', 'replace', None),
         ('ProductDescription', 'text', 'replace', None),
         ('ProductAuthors', 'text', 'replace', None),
         ('DocFormat', 'text', 'replace', None),
         ('ModuleLevelCode', 'code', 'insertAfter', ''),
         ('InitializeBegin', 'code', 'insertAfter', '    '),
         ('InitializeEnd', 'code', 'insertBefore', '    '),)

values = {'Header':'header',
          'ProductName':'product_name',
          'ProductDescription':'product_description',
          'ProductAuthors':'authors',
          'DocFormat':'doc_format',
          'ModuleLevelCode':'\nimport mystuff',
          'InitializeBegin':
"""
for i in range(0,10):
   print i""",
          'InitializeEnd':
"""i = i + 2
print i
""",
         }


from difflib import Differ
def diffText(got, expected):
    d = Differ()
    result = list(d.compare(got.split("\n"), expected.split("\n")))
    for line in result:
        if line.startswith("+") or line.startswith("-"):
            print line

class TestTemplate(unittest.TestCase):

    def setUp(self):
        pass

    def test_pythonTemplate(self):
        # make sure the shuffled sequence does not lose any elements
        ct = template.PythonTemplate(code, file_name='__init__.py',
                                   file_type='python', file_intent='code')
        self.assertEqual(ct.properties['file_name'], '__init__.py')
        self.assertEqual(ct.properties['file_type'], 'python')
        self.assertEqual(ct.properties['file_intent'], 'code')

        i = 0
        for c in ct.children:
            if i % 2:
                sl = slots[i/2]
                self.assertEqual(c.properties['slot_name'], sl[0])
                self.assertEqual(c.properties['slot_type'], sl[1])
                self.assertEqual(c.properties['slot_replacement'], sl[2])
                self.assertEqual(c.properties['indentation'], sl[3])

        self.assertEqual(len(ct.gatherSlots()), len(slots))


    def test_iter(self):
        # make sure the shuffled sequence does not lose any elements
        ct = template.PythonTemplate(code, file_name='__init__.py',
                                   file_type='python', file_intent='code')
        self.assertEqual(len(iter(ct)), 2*len(slots))

    def test_gather(self):
        # collect all the slots
        ct = template.PythonTemplate(code, file_name='__init__.py', file_type='python', file_intent='code')
        assert len(ct.gatherSlots()) == len(slots)

    def test_render(self):
        out = StringIO.StringIO()
        ct = template.PythonTemplate(code, file_name='__init__.py', file_type='python', file_intent='code')
        rendered_code = """#{Header, text, replace}#
\"\"\"
#{ProductName, text, replace}#

#{ProductDescription, text, replace}#

$Id: $
\"\"\"

__authors__ = '#{ProductAuthors, text, replace}#',
__docformat__ = '#{DocFormat, text, replace}#',

#{ModuleLevelCode, code, insertAfter}#

def initialize(context):
    #{InitializeBegin, code, insertAfter}#
    #{InitializeEnd, code, insertBefore}#
    pass

"""
        ct.render({}, out)
        diffText(out.getvalue(), rendered_code)
        self.assertEqual(out.getvalue(),rendered_code)

        rendered_code = """header
\"\"\"
product_name

product_description

$Id: $
\"\"\"

__authors__ = 'authors',
__docformat__ = 'doc_format',

#{ModuleLevelCode, code, insertAfter}#
import mystuff

def initialize(context):
    #{InitializeBegin, code, insertAfter}#
    for i in range(0,10):
       print i
    i = i + 2
    print i
    #{InitializeEnd, code, insertBefore}#
    pass

"""
        out = StringIO.StringIO()
        ct.render(values, out)
        self.assertEqual(out.getvalue(),rendered_code)

    def test_xmltemplate(self):
        values = {'myslot1':'\n<mytag />',
                  'myslot2':
"""<someblock>
  <someotherblock />
</someblock>"""
                  }

        rendered_xml = """
<myxml>
  <sometag />
  <block>
    <!--SLOT:myslot1, code, insertAfter-->
    <mytag />
    <someblock>
      <someotherblock />
    </someblock>
  </block>
</myxml>
"""

        out = StringIO.StringIO()
        xt = template.XMLTemplate(xml, file_name='test.xml', file_type='xml', file_intent='config')
        xt.render(values, out)
        diffText(out.getvalue(), rendered_xml)
        self.assertEqual(out.getvalue(), rendered_xml)


    def test_dom(self):
        xt = template.XMLTemplate(xml, file_name='test.xml', file_type='xml', file_intent='config')
        slot1 = xt.children[1]
        self.assertEqual(slot1.getName(), 'myslot1')
        new_slot = template.Slot('new_slot', 'code', 'replace', '', ('#{', '}#'))
        slot1.insertBefore(new_slot)
        self.assertEqual(xt.children[1].getName(), 'new_slot')

    def test_static(self):
        data = "this is a test"
        t = template.FileTemplate(data)
        assert t.children
        assert t.children[0].getText() == data

    def test_fill(self):
        out = StringIO.StringIO()
        data = {'adverb' : "silly"}

        text = "This is a #{adverb,text,replace}# madlib like thing"
        fragment = template.FileTemplate(text)
        tmpl = template.FileTemplate("Running: #{test,code,replace}#")
        tmpl.getSlotByName("test").fill(fragment)

        tmpl.render(data, out)
        assert "silly" in out.getvalue()
        assert "#{" not in out.getvalue()


def suite():
    return unittest.makeSuite(TestTemplate)

if __name__ == '__main__':
    unittest.main()
